-- Simple Combat Alerts - By Dio (v1.0.5)
-- http://www.esoui.com/downloads/author-1518.html

local SCA = {
	name = "SimpleCombatAlerts",
	title = "SCA",
	dbVersion = 1,
	alerts = {
		-- Priority: BLOCK -> OFF BALANCE -> INTERRUPT -> DODGE
		{key = "block", tipId = 1, label = "BLOCK"},
		{key = "offBalance", tipId = 2, label = "OFF BALANCE"},
		{key = "interrupt", tipId = 3, label = "INTERRUPT"},
		{key = "dodge", tipId = 4, label = "DODGE"}
	}
}

local LMP = LibStub("LibMediaProvider-1.0")

function SCA:Init()
	local defaults = {alerts = {}}

	for _, alert in ipairs(self.alerts) do
		defaults.alerts[alert.key] = {
			show = true,
			label = alert.label,
			color = {1, 0, 0, 1},
			size = 64,
			face = "Univers 67",
			style = "soft-shadow-thin"
		}
	end

	self.db = ZO_SavedVars:New("SCA_DB", self.dbVersion, nil, defaults)

	self:MakeConfig()
	self:MakeControls()

	EVENT_MANAGER:RegisterForEvent(self.name, EVENT_DISPLAY_ACTIVE_COMBAT_TIP, function(_, tipId)
		ZO_ActiveCombatTips:SetHidden(true)
		self.controls.tlw:BringWindowToTop()
		self:ToggleAlert(tipId, true)
	end)

	EVENT_MANAGER:RegisterForEvent(self.name, EVENT_REMOVE_ACTIVE_COMBAT_TIP, function(_, tipId)
		self:ToggleAlert(tipId, false)
	end)

	EVENT_MANAGER:RegisterForEvent(self.name, EVENT_SCREEN_RESIZED, function()
		self:AnchorTLW()
	end)
end

function SCA:GetFontString(font)
	local format = "%s|%d"

	if font.style ~= "none" then
		format = format .. "|%s"
	end

	return format:format(LMP:Fetch(LMP.MediaType.FONT, font.face), font.size, font.style)
end

function SCA:UpdateLabel(label, config, hide)
	local fontString = self:GetFontString(config)
	label:SetFont(fontString)
	label:SetColor(unpack(config.color))
	label:SetText(hide and "" or config.label)
end

function SCA:UpdateAlert(alert, hide)
	self:UpdateLabel(self.controls.alert, self.db.alerts[alert], hide)
end

function SCA:MakeConfig()
	local panel = LibStub("EasyLAM-1.0"):MakePanel(self.title, self.title)

	panel:AddHeader("Options")

	panel:AddButton(
		"Lock / Unlock",
		"Toggle the ability to move the alert.",
		function()
			self:ToggleAlertLock()
		end
	)

	panel:AddButton(
		"Reset Position",
		"Reset the frame back to its default position. The default has the added bonus of enforcing its position regardless of UI scale or resolution changes.",
		function()
			self:AnchorTLW(true)
		end
	)

	for _, alert in ipairs(self.alerts) do
		local db = self.db.alerts[alert.key]
		local header = panel:AddHeader(alert.label):GetNamedChild("Label")

		local updateLabel = function()
			self:UpdateLabel(header, db)
		end

		updateLabel()

		panel:AddCheckbox(
			"Alert:  " .. alert.label,
			"",
			function()
				return db.show
			end,
			function(show)
				db.show = show
			end
		)

		panel:AddEditBox(
			"Label",
			"",
			false,
			function()
				return db.label or alert.label
			end,
			function(label)
				db.label = label == "" and alert.label or label
				updateLabel()
			end
		)

		panel:AddSlider(
			"Size",
			"",
			24,
			72,
			1,
			function()
				return db.size
			end,
			function(size)
				db.size = size
				updateLabel()
			end
		)

		panel:AddDropdown(
			"Font Face",
			"",
			LMP:List(LMP.MediaType.FONT),
			function()
				return db.face
			end,
			function(face)
				db.face = face
				updateLabel()
			end
		)

		panel:AddDropdown(
			"Style",
			"",
			{"none", "soft-shadow-thin", "soft-shadow-thick", "shadow"},
			function()
				return db.style
			end,
			function(style)
				db.style = style
				updateLabel()
			end
		)

		panel:AddColorPicker(
			"Color",
			"",
			function()
				return unpack(db.color)
			end,
			function(r, g, b, a)
				db.color = {r, g, b, a}
				updateLabel()
			end
		)
	end
end

function SCA:ToggleAlert(tipId, show)
	for _, alert in ipairs(self.alerts) do
		if alert.tipId == tipId then
			if self.db.alerts[alert.key].show then
				if self.currentTip and self.currentTip < tipId then
					-- Skip this tip; a higher priority tip is showing.
					return
				end

				self:UpdateAlert(alert.key, not show)
				self.currentTip = show and tipId or nil
			end

			break
		end
	end
end

function SCA:ToggleAlertLock()
	self.controls.tlw.isMovable = not self.controls.tlw.isMovable
	self.controls.tlw:SetMovable(true)
	self:UpdateAlert("block", not self.controls.tlw.isMovable)
end

function SCA:AnchorTLW(reset)
	local a, b, x, y = CENTER, TOP, 0, GuiRoot:GetHeight() / 4

	if reset then
		self.db.tlwPos = nil
	elseif self.db.tlwPos then
		a, b, x, y = unpack(self.db.tlwPos)
	end

	self.controls.tlw:ClearAnchors()
	self.controls.tlw:SetAnchor(a, GuiRoot, b, x, y)
end

function SCA:MakeControls()
	local wm = GetWindowManager()
	self.controls = {}

	self.controls.tlw = wm:CreateTopLevelWindow(self.name)
	self.controls.tlw:SetClampedToScreen(true)
	self.controls.tlw:SetDrawLayer(DL_BACKGROUND)
	self.controls.tlw:SetDrawTier(DT_LOW)
	self.controls.tlw:SetDrawLevel(1)
	self.controls.tlw:SetResizeToFitDescendents(true)
	self.controls.tlw:SetMouseEnabled(true)
	self.controls.tlw.isMovable = false

	self.controls.tlw:SetHandler("OnMouseUp", function(self)
		self:StopMovingOrResizing()
		local _, a, _, b, x, y = self:GetAnchor()
		SCA.db.tlwPos = {a, b, x, y}
	end)

	self:AnchorTLW()

	self.controls.alert = wm:CreateControl(nil, self.controls.tlw, CT_LABEL)
	self.controls.alert:SetAnchor(CENTER, self.controls.tlw)
end

EVENT_MANAGER:RegisterForEvent(SCA.name, EVENT_ADD_ON_LOADED, function(_, name)
	if name == SCA.name then
		EVENT_MANAGER:UnregisterForEvent(SCA.name, EVENT_ADD_ON_LOADED)
		SCA:Init()
	end
end)
